import 'dart:io';
import 'package:args/args.dart';
import 'package:wife/resources/session/package_info.dart';
import "package:wife/resources/template/controller.dart";
import 'package:wife/resources/template/view.dart';
import 'package:wife/shared/extension/clean_filename.dart';
import "package:wife/shared/helper/template/template.dart";
import 'package:path/path.dart';

ArgResults argResults;
Map getArgumentsMap(List<String> args) {
  Map res = {};

  for (var i = 0; i < args.length; i++) {
    var arg = args[i];
    if (arg.startsWith("-")) {
      var key = arg;
      var value = args[i + 1];

      key = key.replaceAll("-", "");
      res[key] = value;
    }
  }

  return res;
}

void doGenerateDefinedTemplate(String templateName) async {
  var file = await File("defined_template/${templateName}.dartx");
  if (!file.existsSync()) return;
  var template = await file.readAsString();
  var fileName = "lib/resources/template/${templateName}.dart";

  template = template.replaceAll("\$", "\\\$");

  template =
      "var ${basename(file.path).split(".")[0]}Template = '''\n${template}\n''';";

  Template.create(fileName, template);
}

void generateDefinedTemplate() async {
  await doGenerateDefinedTemplate("view");
  await doGenerateDefinedTemplate("controller");
}

void createController() async {
  var template = controllerTemplate;
  var outputName = "generated/test/controller.dart";
  await Template.create(outputName, template);
}

void createView() async {
  var template = viewTemplate;
  var outputName = "generated/test/view.dart";
  await Template.create(outputName, template);
}

void createImport() async {
  List importScriptList = [];
  List deepImportScriptList = [];

  var dir = Directory("lib");
  List contents = dir.listSync(recursive: true);

  for (var fileOrDir in contents) {
    if (fileOrDir is File) {
      File file = fileOrDir;

      if (file.path.contains("main.dart")) {
        continue;
      }

      if (file.isDartFile()) {
        //check file content
        var content = file.readAsStringSync();
        var headerLines = content.split("class")[0].split("\n");

        var hasMaterial = false;
        var hasCore = false;
        // var hasAppImport = false;
        var hasDeppImport = false;

        for (var i = 0; i < headerLines.length; i++) {
          var line = headerLines[i];
          if (line.startsWith("import")) {
            headerLines.removeAt(i);

            if (line.contains("$packageName/core.dart")) {
              hasCore = true;
            }

            if (line.contains("flutter/material.dart")) {
              hasMaterial = true;
            }

            // if (line.contains("package:$packageName")) {
            //   hasAppImport = true;
            // }

            if (!(line.contains("$packageName/core.dart") ||
                line.contains("flutter/material.dart") ||
                line.contains("package:$packageName"))) {
              hasDeppImport = true;
              deepImportScriptList.add(line);
            }
            /*
          import 'package:example_app/core.dart';
          import 'package:flutter/material.dart';
          */
          }
        }

        if (hasCore || hasDeppImport) {
          headerLines.add("import 'package:$packageName/core.dart';");
        }

        if (hasMaterial) {
          headerLines.add("import 'package:flutter/material.dart';");
        }

        var bodyIndex = content.indexOf("class");
        var bodyContent = bodyIndex != -1 ? content.substring(bodyIndex) : "";
        var contents = headerLines.join("\n") + bodyContent;
        file.writeAsStringSync(contents);
        //--- check file content

        importScriptList.add(file.getExportScriptFromFileName());
      }
    }
  }

  var outputName = "lib/core.dart";
  var packageImport = await getPackageImport(
    deepImportScriptList: deepImportScriptList,
  );

  var content = "${packageImport}\n\n${importScriptList.join("\n")}";
  await Template.create(outputName, content);
}

Future<String> getPackageImport({
  List deepImportScriptList = const [],
}) async {
  List importScriptList = [];
  var content = await File("pubspec.yaml").readAsString();

  var start = 'dependencies:';
  var end = 'dev_dependencies:';

  var startIndex = content.indexOf(start);
  var endIndex = content.indexOf(end);
  var result = content.substring(startIndex + start.length, endIndex).trim();

  var arr = result.split("\n");

  var excludeList = [
    "flutter",
    "sdk",
    "git",
    "url",
  ];

  for (var i = 0; i < arr.length; i++) {
    var row = arr[i];

    if (row.contains(":") && !row.contains("#")) {
      var lib = row.split(":")[0].trim();

      if (excludeList.contains(lib)) {
        print("Skip $lib");
        continue;
      }
      ;

      importScriptList.add("export \'package:$lib/$lib.dart\';");
    }
  }

  //Add deepImportScriptList
  deepImportScriptList.forEach((line) {
    importScriptList.add(
        line.toString().replaceFirst("import", "export").replaceAll("\"", "'"));
  });
  //---

  return importScriptList.toSet().toList().join("\n");
}
